﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Styx;
using Styx.Common;
using Styx.CommonBot;
using Styx.CommonBot.Routines;
using Styx.Helpers;
using Styx.TreeSharp;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using Styx.MemoryManagement;
using Action = Styx.TreeSharp.Action;


namespace Tricks
{
    public partial class Routine
    {

        public bool IsAttackable(WoWUnit unit)
        {
            if (unit == null)
                return false;

            if (!unit.CanSelect)
                return false;

            if (!unit.Attackable)
                return false;

            if (unit.IsDead)
                return false;

            //if (IsImmune(unit))
            //    return false;

            //if (IsInCC(unit))
            //    return false;

            if (unit.IsMe)
                return false;

            if (unit.IsHostile)
                return true;
            
            if (!unit.IsFriendly)
                return true;

            return false;
        }
        public IEnumerable<WoWUnit> EnemiesNeedingKick
        {
            get
            {

                try
                {
                    var ret = ObjectManager.GetObjectsOfType<WoWUnit>(true, false).Where(
                        u =>
                            u.IsAlive &&
                            !u.IsMe &&
                            IsAttackable(u) &&
                            !u.IsFriendly &&
                            !u.IsPet &&
                            u.IsWithinMeleeRange &&
                            u.IsCasting &&
                            u.CurrentCastTimeLeft.TotalMilliseconds < CastTimeLeftInMS
                            //u.CastingSpell.Name == "Fear" ||
                            //u.CastingSpell.Name == "Polymorph" ||
                            //u.CastingSpell.Name == "Cyclone" ||
                            //u.CastingSpell.Name == "Repentence" ||
                            //u.CastingSpell.Name == "Hex" ||
                            //u.IsCastingHealingSpell
                                  );

                    return ret.ToList();
                }
                catch (NullReferenceException)
                {
                    return new List<WoWUnit>();
                }
            }
        }
        public IEnumerable<WoWUnit> EnemiesNeedingGougeInterrupt
        {
            get
            {

                try
                {
                    var ret = ObjectManager.GetObjectsOfType<WoWUnit>(true, false).Where(
                        u =>
                            u.IsAlive &&
                            !u.IsMe &&
                            IsAttackable(u) &&
                            !u.IsFriendly &&
                            !u.IsPet &&
                            u.IsWithinMeleeRange &&
                            StyxWoW.Me.IsFacing(u) &&
                            u.IsCasting &&
                            u.CurrentCastTimeLeft.TotalMilliseconds < CastTimeLeftInMS && (
                            u.CastingSpell.Name == "Fear" ||
                            u.CastingSpell.Name == "Polymorph" ||
                            u.CastingSpell.Name == "Cyclone" ||
                            u.CastingSpell.Name == "Repentence" ||
                            u.CastingSpell.Name == "Hex" ||
                            u.IsCastingHealingSpell
                            )
                                  );

                    return ret.ToList();
                }
                catch (NullReferenceException)
                {
                    return new List<WoWUnit>();
                }
            }
        }
        public IEnumerable<WoWUnit> FindStealthy
        {
            get
            {

                try
                {
                    var ret = ObjectManager.GetObjectsOfType<WoWUnit>(true, false).Where(
                        u =>
                            IsAttackable(u) &&
                            u.IsAlive &&
                            !u.IsMe &&
                            !u.IsPet &&
                            (u.IsHumanoid || u.IsBeast || u.IsDemon || u.IsDragon) &&
                            u.Distance <= 10 &&
                            u.IsStealthed &&
                            !u.ActiveAuras.ContainsKey("Sap")
                     );

                    return ret.ToList();
                }
                catch (NullReferenceException)
                {
                    return new List<WoWUnit>();
                }
            }
        }
        public IEnumerable<WoWUnit> EnemiesTargetingMe
        {
            get
            {

                try
                {
                    var ret = ObjectManager.GetObjectsOfType<WoWUnit>(true, false).Where(
                        u =>
                            u.IsHostile &&
                            u.Distance <= 40 &&
                            u.IsAlive &&
                            !u.IsMe &&
                            !u.IsPet &&
                            u.IsTargetingMeOrPet
                     );

                    return ret.ToList();
                }
                catch (NullReferenceException)
                {
                    return new List<WoWUnit>();
                }
            }
        }
        public IEnumerable<WoWPlayer> FriendsNeedingHelp30
        {
            get
            {
                try
                {
                    var ret = ObjectManager.GetObjectsOfType<WoWPlayer>(true, false).Where(
                        u =>
                            u.IsAlive &&
                            !u.IsMe &&
                            u.IsFriendly &&
                            u.HealthPercent <= 25 &&
                            u.InLineOfSpellSight &&
                            u.Distance <= 30
                    );

                    return ret.ToList();
                }
                catch (NullReferenceException)
                {
                    return new List<WoWPlayer>();
                }
            }
        }
        public bool IsCrowdControlled(WoWUnit unit)
        {
            if (unit != null)
            {
                Dictionary<string, WoWAura>.ValueCollection auras = unit.Auras.Values;

                return auras.Any(a =>
                           a.IsHarmful && (
                           a.Spell.Mechanic == WoWSpellMechanic.Banished ||
                           a.Spell.Mechanic == WoWSpellMechanic.Disoriented ||
                           a.Spell.Mechanic == WoWSpellMechanic.Charmed ||
                           a.Spell.Mechanic == WoWSpellMechanic.Horrified ||
                           a.Spell.Mechanic == WoWSpellMechanic.Incapacitated ||
                           a.Spell.Mechanic == WoWSpellMechanic.Polymorphed ||
                           a.Spell.Mechanic == WoWSpellMechanic.Sapped ||
                           a.Spell.Mechanic == WoWSpellMechanic.Shackled ||
                           a.Spell.Mechanic == WoWSpellMechanic.Asleep ||
                           a.Spell.Mechanic == WoWSpellMechanic.Frozen ||
                           a.Spell.Mechanic == WoWSpellMechanic.Invulnerable ||
                           a.Spell.Mechanic == WoWSpellMechanic.Invulnerable2 ||
                           a.Spell.Mechanic == WoWSpellMechanic.Turned ||
                           a.Spell.Mechanic == WoWSpellMechanic.Fleeing ||


                           // Really want to ignore hexed mobs.
                           a.Spell.Name == "Hex"));
            }
            return false;
        }
        public bool IsImmune(WoWUnit unit)
        {
            if (unit != null)
            {
                if (unit.ActiveAuras.ContainsKey("Ice Block") ||
                    unit.ActiveAuras.ContainsKey("Hand of Protection") ||
                    unit.ActiveAuras.ContainsKey("Divine Shield") ||
                    unit.ActiveAuras.ContainsKey("Deterrence") ||
                    unit.ActiveAuras.ContainsKey("Cyclone"))
                    return true;
            }
            return false;
        }
        private int MyDebuffTime(String spellName, WoWUnit unit)
        {
            using (StyxWoW.Memory.AcquireFrame())
            {
                if (unit.HasAura(spellName))
                {
                    var auras = unit.GetAllAuras();
                    foreach (var a in auras)
                    {
                        if (a.Name == spellName && a.CreatorGuid == StyxWoW.Me.Guid)
                        {
                            return a.TimeLeft.Seconds;
                        }
                    }
                }
            }
            return 0;
        }
        public static Composite RunMacroText(string macro, CanRunDecoratorDelegate cond)
        {
            return new Decorator(
                       cond,
                       new PrioritySelector(
                           new Action(a => Styx.WoWInternals.Lua.DoString("RunMacroText(\"" + RealLuaEscape(macro) + "\")"))));
        }
        public static string RealLuaEscape(string luastring)
        {
            var bytes = Encoding.UTF8.GetBytes(luastring);
            return bytes.Aggregate(String.Empty, (current, b) => current + ("\\" + b));
        }
        public bool IsInCC(WoWUnit unit)
        {
            if (unit != null)
            {
                if (unit.ActiveAuras.ContainsKey("Repentance") ||
                    unit.ActiveAuras.ContainsKey("Blinding Light") ||
                    unit.ActiveAuras.ContainsKey("Intimidating Shout") ||
                    unit.ActiveAuras.ContainsKey("Wyvern Sting") ||
                    unit.ActiveAuras.ContainsKey("Polymorph") ||
                    unit.ActiveAuras.ContainsKey("Freezing Trap") ||
                    unit.ActiveAuras.ContainsKey("Sap") ||
                    unit.ActiveAuras.ContainsKey("Blind"))
                    return true;
            }
            return false;
        }
        private static bool HasDOT(WoWUnit target)
        {
            if (!target.IsPlayer)
            {
                return false;
            }

            return //DOTs
                   target.HasAura(980) || //Agony
                   target.HasAura(3674) || //Black Arrow
                   target.HasAura(128531) || //Blackout Kick
                   target.HasAura(55078) || //Blood Plague
                   target.HasAura(31803) || //Censure
                   target.HasAura(83853) || //Combustion
                   target.HasAura(172) || //Corruption
                   target.HasAura(770) || //faerie fire
                   target.HasAura(123996) || //Crackling Tiger Lightning
                   target.HasAura(122233) || //Crimson Tempest
                   target.HasAura(2818) || //Deadly Poison
                   target.HasAura(43265) || //Death and Decay
                   target.HasAura(115767) || //Deep Wounds
                   target.HasAura(2944) || //Devouring Plague
                   target.HasAura(603) || //Doom
                   target.HasAura(53301) || //Explosive Shot
                   target.HasAura(8050) || //Flame Shock
                   target.HasAura(2120) || //Flamestrike
                   target.HasAura(113092) || //Frost Bomb
                   target.HasAura(55095) || //Frost Fever
                   target.HasAura(703) || //Garrote
                   target.HasAura(14914) || //Holy Fire
                   target.HasAura(33745) || //Lacerate
                   target.HasAura(44457) || //Living Bomb
                   target.HasAura(103103) || //Malefic Grasp
                   target.HasAura(8921) || //Moonfire
                   target.HasAura(114923) || //Nether Tempest
                   target.HasAura(63468) || //Piercing Shots
                   target.HasAura(9007) || //Pounce Bleed
                   target.HasAura(11366) || //Pyroblast
                   target.HasAura(1822) || //Rake
                   target.HasAura(1079) || //Rip
                   target.HasAura(1943) || //Rupture
                   target.HasAura(118253) || //Serpent Sting
                   target.HasAura(589) || //Shadow Word: Pain
                   target.HasAura(93402) || //Sunfire
                   target.HasAura(77758) || //Thrash
                   target.HasAura(106830) || //Thrash
                   target.HasAura(31117) || //Unstable Affliction
                   target.HasAura(91075) || //Vengeful Wisp
                   target.HasAura(91076); //Vengeful Wisp

        }
        public IEnumerable<WoWUnit> EnemyCastingDistance10
        {
            get
            {
                try
                {
                    var ret = ObjectManager.GetObjectsOfType<WoWUnit>(true, false).Where(
                        u =>
                            IsAttackable(u) &&
                            u.IsAlive &&
                            !u.IsMe &&
                            u.IsHostile &&
                            !u.IsPet &&
                            u.InLineOfSpellSight &&
                            u.IsCasting &&
                            u.Distance <= 10
                     );

                    return ret.ToList();
                }
                catch (NullReferenceException)
                {
                    return new List<WoWUnit>();
                }
            }
        }
        public IEnumerable<WoWPlayer> FriendsNeedingTricks100
        {
            get
            {
                try
                {
                    var ret = ObjectManager.GetObjectsOfType<WoWPlayer>(true, false).Where(
                        u =>
                            u.IsAlive &&
                            !u.IsMe &&
                            !u.IsPet &&
                            u.IsFriendly &&
                            u.IsInMyPartyOrRaid &&
                            u.InLineOfSpellSight &&
                            u.Distance <= 100
                    );

                    return ret.ToList();
                }
                catch (NullReferenceException)
                {
                    return new List<WoWPlayer>();
                }
            }
        }
        public IEnumerable<WoWUnit> EnemyUnitsInSap
        {
            get
            {
                try
                {
                    var ret = ObjectManager.GetObjectsOfType<WoWUnit>(true, false).Where(
                        u =>
                            IsAttackable(u) &&
                            u.ActiveAuras.ContainsKey("Sap") &&
                            u.Distance <= 60
                    );

                    return ret.ToList();
                }
                catch (NullReferenceException)
                {
                    return new List<WoWUnit>();
                }
            }
        }
        public bool HasDOTs(WoWUnit u)
        {
            return
                u.ActiveAuras.Any(a => 
                    (a.Value.ApplyAuraType == WoWApplyAuraType.PeriodicDamage ||
                    a.Value.ApplyAuraType == WoWApplyAuraType.PeriodicDamagePercent ||
                    a.Value.ApplyAuraType == WoWApplyAuraType.PeriodicLeech)
                );
        }

        #region Tricks

        static string[] CrowdControlSpellNames = new string[] { "Chaos Bolt", "Cyclone", "Entangling Roots", "Fear", "Hex", "Polymorph" };
        static IEnumerable<WoWPlayer> EnemiesCastingCrowdControl60 =
            ObjectManager.GetObjectsOfType<WoWPlayer>(true, false)
            .Where(p => p.Distance <= 60 && p.IsHostile && p.IsCasting && CrowdControlSpellNames.Contains(p.CastingSpell.Name)).ToList();

        static IEnumerable<WoWPlayer> nearbyFriendlies20 =
            ObjectManager.GetObjectsOfType<WoWPlayer>(true, false)
            .Where(p => p.Distance <= 20 && p.IsFriendly && p.IsAlive).ToList();

        IEnumerable<WoWApplyAuraType> _dpsIncreaseAurasTypes = new List<WoWApplyAuraType>()
        {
            WoWApplyAuraType.ModAllCritChance,
            WoWApplyAuraType.ModAttackerMeleeCritChance,
            WoWApplyAuraType.ModAttackerRangedCritChance,
            WoWApplyAuraType.ModAttackerSpellAndWeaponCritChance,
            WoWApplyAuraType.ModAttackerSpellCritChance,
            WoWApplyAuraType.ModCritPercent,
            WoWApplyAuraType.ModCritDamageBonusMelee,
            WoWApplyAuraType.ModDamageDone,
            WoWApplyAuraType.ModDamagePercentDone,
            WoWApplyAuraType.ModSpellCritChance,
            WoWApplyAuraType.ModDamagePercentDone,
            WoWApplyAuraType.ModAttackPower,
            WoWApplyAuraType.ModAttackPowerPct
            
        };
        #endregion
        public bool IsMovementImpaired(WoWUnit unit)
        {
            if (unit != null)
            {
                Dictionary<string, WoWAura>.ValueCollection auras = unit.Auras.Values;

                return auras.Any(a =>
                           a.IsHarmful && (a.Spell.Mechanic == WoWSpellMechanic.Snared || a.Spell.Mechanic == WoWSpellMechanic.Rooted));
            }
            return false;
        }
        public bool IsCaster(WoWUnit unit)
        {
            if (unit != null)
            {

                if (
                    unit.Class == WoWClass.Priest ||
                    unit.Class == WoWClass.Druid ||
                    (unit.Class == WoWClass.Monk && unit.CurrentMana >= 100000) ||
                    unit.Class == WoWClass.Mage ||
                    unit.Class == WoWClass.Warlock ||
                    (unit.Class == WoWClass.Paladin && unit.CurrentMana >= 100000) ||
                    (unit.Class == WoWClass.Shaman && unit.CurrentMana >= 100000)
                    )
                return true;
            }
            return false;
        }
        public bool IsStunned(WoWUnit unit)
        {
            if (unit != null)
            {
                Dictionary<string, WoWAura>.ValueCollection auras = unit.Auras.Values;

                return auras.Any(a =>
                           a.IsHarmful && (
                           a.Spell.Mechanic == WoWSpellMechanic.Stunned));
            }
            return false;
        }
        public bool InterruptImmune(WoWUnit unit)
        {
            if (unit != null)
            {
                if (unit.HasAura(89489) || //Glyph of Inner Focus
                    unit.HasAura("Unending Resolve") ||
                    unit.HasAura("Devotion Aura")) 
                   return true;
            }
            return false;
        }

    }
}
